pre { line-height: 125%; }
td.linenos .normal { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }
span.linenos { color: inherit; background-color: transparent; padding-left: 5px; padding-right: 5px; }
td.linenos .special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }
span.linenos.special { color: #000000; background-color: #ffffc0; padding-left: 5px; padding-right: 5px; }
.codehilite .hll, .highlight .hll { background-color: #49483e }
.codehilite , .highlight { background: #272822; color: #F8F8F2 }
.codehilite .c, .highlight .c { color: #959077 } /* Comment */
.codehilite .err, .highlight .err { color: #ED007E; background-color: #1E0010 } /* Error */
.codehilite .esc, .highlight .esc { color: #F8F8F2 } /* Escape */
.codehilite .g, .highlight .g { color: #F8F8F2 } /* Generic */
.codehilite .k, .highlight .k { color: #66D9EF } /* Keyword */
.codehilite .l, .highlight .l { color: #AE81FF } /* Literal */
.codehilite .n, .highlight .n { color: #F8F8F2 } /* Name */
.codehilite .o, .highlight .o { color: #FF4689 } /* Operator */
.codehilite .x, .highlight .x { color: #F8F8F2 } /* Other */
.codehilite .p, .highlight .p { color: #F8F8F2 } /* Punctuation */
.codehilite .ch, .highlight .ch { color: #959077 } /* Comment.Hashbang */
.codehilite .cm, .highlight .cm { color: #959077 } /* Comment.Multiline */
.codehilite .cp, .highlight .cp { color: #959077 } /* Comment.Preproc */
.codehilite .cpf, .highlight .cpf { color: #959077 } /* Comment.PreprocFile */
.codehilite .c1, .highlight .c1 { color: #959077 } /* Comment.Single */
.codehilite .cs, .highlight .cs { color: #959077 } /* Comment.Special */
.codehilite .gd, .highlight .gd { color: #FF4689 } /* Generic.Deleted */
.codehilite .ge, .highlight .ge { color: #F8F8F2; font-style: italic } /* Generic.Emph */
.codehilite .ges, .highlight .ges { color: #F8F8F2; font-weight: bold; font-style: italic } /* Generic.EmphStrong */
.codehilite .gr, .highlight .gr { color: #F8F8F2 } /* Generic.Error */
.codehilite .gh, .highlight .gh { color: #F8F8F2 } /* Generic.Heading */
.codehilite .gi, .highlight .gi { color: #A6E22E } /* Generic.Inserted */
.codehilite .go, .highlight .go { color: #66D9EF } /* Generic.Output */
.codehilite .gp, .highlight .gp { color: #FF4689; font-weight: bold } /* Generic.Prompt */
.codehilite .gs, .highlight .gs { color: #F8F8F2; font-weight: bold } /* Generic.Strong */
.codehilite .gu, .highlight .gu { color: #959077 } /* Generic.Subheading */
.codehilite .gt, .highlight .gt { color: #F8F8F2 } /* Generic.Traceback */
.codehilite .kc, .highlight .kc { color: #66D9EF } /* Keyword.Constant */
.codehilite .kd, .highlight .kd { color: #66D9EF } /* Keyword.Declaration */
.codehilite .kn, .highlight .kn { color: #FF4689 } /* Keyword.Namespace */
.codehilite .kp, .highlight .kp { color: #66D9EF } /* Keyword.Pseudo */
.codehilite .kr, .highlight .kr { color: #66D9EF } /* Keyword.Reserved */
.codehilite .kt, .highlight .kt { color: #66D9EF } /* Keyword.Type */
.codehilite .ld, .highlight .ld { color: #E6DB74 } /* Literal.Date */
.codehilite .m, .highlight .m { color: #AE81FF } /* Literal.Number */
.codehilite .s, .highlight .s { color: #E6DB74 } /* Literal.String */
.codehilite .na, .highlight .na { color: #A6E22E } /* Name.Attribute */
.codehilite .nb, .highlight .nb { color: #F8F8F2 } /* Name.Builtin */
.codehilite .nc, .highlight .nc { color: #A6E22E } /* Name.Class */
.codehilite .no, .highlight .no { color: #66D9EF } /* Name.Constant */
.codehilite .nd, .highlight .nd { color: #A6E22E } /* Name.Decorator */
.codehilite .ni, .highlight .ni { color: #F8F8F2 } /* Name.Entity */
.codehilite .ne, .highlight .ne { color: #A6E22E } /* Name.Exception */
.codehilite .nf, .highlight .nf { color: #A6E22E } /* Name.Function */
.codehilite .nl, .highlight .nl { color: #F8F8F2 } /* Name.Label */
.codehilite .nn, .highlight .nn { color: #F8F8F2 } /* Name.Namespace */
.codehilite .nx, .highlight .nx { color: #A6E22E } /* Name.Other */
.codehilite .py, .highlight .py { color: #F8F8F2 } /* Name.Property */
.codehilite .nt, .highlight .nt { color: #FF4689 } /* Name.Tag */
.codehilite .nv, .highlight .nv { color: #F8F8F2 } /* Name.Variable */
.codehilite .ow, .highlight .ow { color: #FF4689 } /* Operator.Word */
.codehilite .pm, .highlight .pm { color: #F8F8F2 } /* Punctuation.Marker */
.codehilite .w, .highlight .w { color: #F8F8F2 } /* Text.Whitespace */
.codehilite .mb, .highlight .mb { color: #AE81FF } /* Literal.Number.Bin */
.codehilite .mf, .highlight .mf { color: #AE81FF } /* Literal.Number.Float */
.codehilite .mh, .highlight .mh { color: #AE81FF } /* Literal.Number.Hex */
.codehilite .mi, .highlight .mi { color: #AE81FF } /* Literal.Number.Integer */
.codehilite .mo, .highlight .mo { color: #AE81FF } /* Literal.Number.Oct */
.codehilite .sa, .highlight .sa { color: #E6DB74 } /* Literal.String.Affix */
.codehilite .sb, .highlight .sb { color: #E6DB74 } /* Literal.String.Backtick */
.codehilite .sc, .highlight .sc { color: #E6DB74 } /* Literal.String.Char */
.codehilite .dl, .highlight .dl { color: #E6DB74 } /* Literal.String.Delimiter */
.codehilite .sd, .highlight .sd { color: #E6DB74 } /* Literal.String.Doc */
.codehilite .s2, .highlight .s2 { color: #E6DB74 } /* Literal.String.Double */
.codehilite .se, .highlight .se { color: #AE81FF } /* Literal.String.Escape */
.codehilite .sh, .highlight .sh { color: #E6DB74 } /* Literal.String.Heredoc */
.codehilite .si, .highlight .si { color: #E6DB74 } /* Literal.String.Interpol */
.codehilite .sx, .highlight .sx { color: #E6DB74 } /* Literal.String.Other */
.codehilite .sr, .highlight .sr { color: #E6DB74 } /* Literal.String.Regex */
.codehilite .s1, .highlight .s1 { color: #E6DB74 } /* Literal.String.Single */
.codehilite .ss, .highlight .ss { color: #E6DB74 } /* Literal.String.Symbol */
.codehilite .bp, .highlight .bp { color: #F8F8F2 } /* Name.Builtin.Pseudo */
.codehilite .fm, .highlight .fm { color: #A6E22E } /* Name.Function.Magic */
.codehilite .vc, .highlight .vc { color: #F8F8F2 } /* Name.Variable.Class */
.codehilite .vg, .highlight .vg { color: #F8F8F2 } /* Name.Variable.Global */
.codehilite .vi, .highlight .vi { color: #F8F8F2 } /* Name.Variable.Instance */
.codehilite .vm, .highlight .vm { color: #F8F8F2 } /* Name.Variable.Magic */
.codehilite .il, .highlight .il { color: #AE81FF } /* Literal.Number.Integer.Long */
